package base

import (
	"github.com/gin-gonic/gin"
	"net/http"
)

/*
{
	"code": 10000, // 程序中的错误码
	"msg": xx,     // 提示信息
	"data": {},    // 数据
}

*/

// ResponseData 统一返回数据
type ResponseData struct {
	Code int         `json:"code"`
	Msg  interface{} `json:"msg"`
	Data interface{} `json:"data,omitempty"`
}

// ResponseErrorWithMsg 返回错误
func ResponseErrorWithMsg(c *gin.Context, code int) {
	c.JSON(http.StatusOK, &ResponseData{
		Code: code,
		Msg:  InternationalizedMsg(c, code),
		Data: nil,
	})
}

// ResponseSuccess 返回正确
func ResponseSuccess(c *gin.Context, data interface{}) {
	c.JSON(http.StatusOK, &ResponseData{
		Code: Success,
		Msg:  InternationalizedMsg(c, Success),
		Data: data,
	})
}

// ResponseErrorMsg 直接返回msg
func ResponseErrorMsg(c *gin.Context, msg string) {
	c.JSON(http.StatusOK, &ResponseData{
		Code: Warning,
		Msg:  msg,
		Data: nil,
	})
}